/*-
 * Copyright (c) 2009 - 2015 Rozhuk Ivan <rozhuk.im@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Rozhuk Ivan <rozhuk.im@gmail.com>
 *
 */

#pragma comment(linker, "/subsystem:Windows,5")
#pragma comment(linker, "/version:5")

#ifdef NDEBUG
	//#pragma comment(linker, "/MERGE:.rdata=.text")
	//#pragma comment(linker, "/FILEALIGN:512 /IGNORE:4078")
	//#pragma comment(linker, "/opt:nowin98")
	//#pragma comment(linker, "/nodefaultlib")
	#pragma comment(linker, "/ENTRY:WinMainEx")
#endif //_DEBUG


#define WINVER       0x501
#define _WIN32_WINNT 0x501
//#define WINVER       0x600
//#define _WIN32_WINNT 0x600
//#define UNICODE
//#define _UNICODE


// Windows Header Files:
#include <windows.h>
#include <shellapi.h>
#include <commctrl.h>
#include <commdlg.h>
#include <Ras.h>
#include <RasError.h>
#include <Rasdlg.h>
//#include <strsafe.h>
//#include <afxres.h>
#include "resource.h"
#include "CRCFileCheck.h"
#include "ReportError.h"


#pragma comment(lib, "comctl32.lib")
#pragma comment(lib, "Rasapi32.lib")
#pragma comment(lib, "Rasdlg.lib")


#define MEMALLOC(Size)			HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, (Size + sizeof(SIZE_T)))
#define MEMREALLOC(Mem, Size)		HeapReAlloc(GetProcessHeap(), (HEAP_ZERO_MEMORY), (LPVOID)Mem, (Size + sizeof(SIZE_T)))
#define MEMFREE(Mem)			if (Mem) { HeapFree(GetProcessHeap(), 0, (LPVOID)Mem); Mem = NULL; }
#define GET_WINDOW_DATA(hWndDlg)	((LPVOID)GetWindowLongPtr(hWndDlg, GWL_USERDATA))
#define SET_WINDOW_DATA(hWndDlg,Data)	((LPVOID)SetWindowLongPtr(hWndDlg, GWL_USERDATA, Data))
#define SIZEOF(X)			(sizeof(X) / sizeof(X[0]))

#define MAX_PATH_LEN			32768


LPCTSTR lpcszMainSectionName = TEXT("Main");
LPCTSTR lpcszWindowSettingsName = TEXT("WindowSettings");

// conn file
LPCTSTR lpcszConnSectionName = TEXT("Connection");
LPCTSTR lpcszConnName = TEXT("ConnectionName");
LPCTSTR lpcszRasEntryName = TEXT("RasEntry");


//    
struct PROCESS_SETTINGS {
	HINSTANCE	hInstance; // * 
};
static PROCESS_SETTINGS psProcessSettings;


BOOL		WritePrivateProfileInt(LPCTSTR lpAppName, LPCTSTR lpKeyName, INT nValue, LPCTSTR lpFileName);
LRESULT CALLBACK WndProcMainDialog(HWND, UINT, WPARAM, LPARAM);
DWORD		AskForConnName(HWND hWndParent, LPWSTR lpszName);
INT_PTR CALLBACK WndProcGetConnName(HWND, UINT, WPARAM, LPARAM);
DWORD		GetConnectionsList(LPRASENTRYNAME *plpRasEntryName, SIZE_T *pdwCount);



#ifdef _DEBUG
	#ifdef UNICODE
		int APIENTRY wWinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPWSTR lpCmdLine,int nCmdShow)
	#else
		int APIENTRY WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow)
	#endif
#else
	void WinMainEx(void)
#endif //_DEBUG
{
	#ifdef NDEBUG
		HINSTANCE	hInstance = GetModuleHandle(NULL);
		HINSTANCE	hPrevInstance = NULL;
		LPTSTR		lpCmdLine = GetCommandLine();
		int		nCmdShow = 0;
	#endif //_DEBUG

	DWORD dwRet = NO_ERROR;

	//    
	dwRet = CRCFileCheck();
	if (dwRet == NO_ERROR) {//     
		MSG msg;
		HWND hMainWindow;

		ZeroMemory(&psProcessSettings, sizeof(psProcessSettings));
		psProcessSettings.hInstance = hInstance;

		//LoadLibrary(TEXT("Riched20.dll"));
		InitCommonControls();

		hMainWindow = CreateDialog(hInstance, MAKEINTRESOURCE(IDD_MAINWINDOW), NULL, (DLGPROC)WndProcMainDialog);
		while (GetMessage(&msg, 0, 0, 0)) {
			if (!IsDialogMessage(hMainWindow, &msg)) {
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
		}
	}

	if (dwRet != NO_ERROR)
		ReportError(dwRet);

#ifdef _DEBUG
	return (dwRet);
#else
	ExitProcess(dwRet);
#endif //_DEBUG
}


BOOL
WritePrivateProfileInt(LPCTSTR lpAppName, LPCTSTR lpKeyName, INT nValue, LPCTSTR lpFileName) {
	TCHAR szBuff[32];

	wsprintf(szBuff, TEXT("%lu"), nValue);

	return (WritePrivateProfileString(lpAppName, lpKeyName, szBuff, lpFileName));
}
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
LRESULT CALLBACK
WndProcMainDialog(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {

	switch (message) {
	case WM_INITDIALOG:
		{
			HICON ico;

			ico = LoadIcon(NULL, MAKEINTRESOURCE(IDI_INFORMATION));
			SendMessage(hWnd, WM_SETICON, 0, (LPARAM)ico);
			SendMessage(hWnd, WM_SETICON, 1, (LPARAM)ico);
			DestroyIcon(ico);

			SendMessage(hWnd, WM_COMMAND, IDC_BUTTON_CONS_REFRESH, 0);
		}
		break;
	case WM_CLOSE:
	case WM_QUIT:
		EndDialog(hWnd, LOWORD(wParam));
		DestroyWindow(hWnd);
		PostQuitMessage(0);
		break;
	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		//case ID_FILE_OPEN:
		//	break;
		//case ID_FILE_SAVE:
		//	break;
		case ID_FILE_SAVEAS:
		case IDC_BUTTON_CONS_SAVEAS:
			{
				WCHAR szEntryName[RAS_MaxEntryName + 1];
				WCHAR szFileName[MAX_PATH_LEN], szFilter[MAX_PATH];
				OPENFILENAME ofn = { 0 };
				DWORD dwEntryInfoSize;
				RASENTRY reEntry = { 0 };

				if (!GetDlgItemTextW(hWnd, IDC_COMBO_CONNECTIONS,
				    szEntryName, SIZEOF(szEntryName)))
					break;
				lstrcpyn(szFileName, szEntryName, RAS_MaxEntryName);
				lstrcpyn(szFilter, TEXT("ini file (*.ini)|*.ini|all files|*.*||"), MAX_PATH);
				for (SIZE_T i = 0; szFilter[i]; i ++) {
					if (szFilter[i] == '|')
						szFilter[i] = 0;
				}

				ofn.lStructSize	= sizeof(OPENFILENAME);
				ofn.hwndOwner = hWnd;
				ofn.hInstance = psProcessSettings.hInstance;
				ofn.lpstrFilter = szFilter;
				//ofn.lpstrCustomFilter = NULL;
				//ofn.nMaxCustFilter = 0;
				//ofn.nFilterIndex = iFilter;
				ofn.lpstrFile = szFileName;
				ofn.nMaxFile = MAX_PATH_LEN;
				//ofn.lpstrFileTitle = NULL;
				//ofn.nMaxFileTitle = 0;
				//ofn.lpstrInitialDir = 0;
				ofn.lpstrTitle = TEXT("Save connection file");
				ofn.Flags = (OFN_DONTADDTORECENT | OFN_ENABLESIZING | OFN_EXPLORER | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT | OFN_PATHMUSTEXIST);
				//ofn.nFileOffset = 0;
				//ofn.nFileExtension = 0;
				ofn.lpstrDefExt = TEXT(".ini");
				//ofn.lCustData = 0;
				//ofn.lpfnHook = 0;
				//ofn.lpTemplateName = NULL;
				//ofn.pvReserved = NULL;
				//ofn.dwReserved = 0;
				//ofn.FlagsEx = 0;

				if (!GetSaveFileName(&ofn))
					break;
				GetDlgItemTextW(hWnd, IDC_COMBO_CONNECTIONS, szEntryName, SIZEOF(szEntryName));
				reEntry.dwSize = sizeof(RASENTRY);
				dwEntryInfoSize = sizeof(RASENTRY);
				if (RasGetEntryProperties(NULL, szEntryName, &reEntry, &dwEntryInfoSize, NULL, NULL) != NO_ERROR)
					break;
				if (reEntry.dwType == RASET_Vpn || reEntry.dwType == RASET_Broadband) {
					WritePrivateProfileString(lpcszConnSectionName, lpcszConnName, szEntryName, szFileName);
					WritePrivateProfileStruct(lpcszConnSectionName, lpcszRasEntryName, &reEntry, sizeof(RASENTRY), szFileName);
				}
			}
			break;
		case ID_FILE_EXIT:
			//EndDialog(hWnd,0);
			SendMessage(hWnd, WM_CLOSE, 0, 0);
			break;
		case ID_HELP_ABOUT:
			MessageBox(hWnd, TEXT("by Rozhuk Ivan 2009 - 2015"),
			    TEXT("About PPP helper configurator"), (MB_OK | MB_ICONINFORMATION));
			//DialogBox(AppSettings.hInst,(LPCTSTR)IDD_ABOUT,AppSettings.hMainWindow,(DLGPROC)WndProcAbout);
			break;
		case IDC_BUTTON_CONS_REFRESH:
			{
				SIZE_T dwCount = 0;
				LPRASENTRYNAME lpRasEntryName = NULL;
				HWND hWndCombo = GetDlgItem(hWnd, IDC_COMBO_CONNECTIONS);
				DWORD dwItem, dwEntryInfoSize;
				RASENTRY reEntry = { 0 };

				if (GetConnectionsList(&lpRasEntryName, &dwCount) != NO_ERROR)
					break;
				SendMessage(hWndCombo, CB_RESETCONTENT, 0, 0);
				//dwItem=SendMessage(hWndCombo, CB_ADDSTRING, 0, (LPARAM)TEXT(""));
				//SendMessage(hWndCombo, CB_SETITEMDATA, dwItem, 0);
				//SendMessage(hWndCombo, CB_SETCURSEL, dwItem, 0);

				for (SIZE_T i = 0; i < dwCount; i ++) {
					reEntry.dwSize = sizeof(RASENTRY);
					dwEntryInfoSize = sizeof(RASENTRY);
					if (RasGetEntryProperties(NULL, lpRasEntryName[i].szEntryName,
					    &reEntry, &dwEntryInfoSize, NULL, NULL) == NO_ERROR) {
						if (reEntry.dwType == RASET_Vpn || reEntry.dwType == RASET_Broadband) {
							dwItem = SendMessage(hWndCombo, CB_ADDSTRING, 0,
							    (LPARAM)lpRasEntryName[i].szEntryName);
							SendMessage(hWndCombo, CB_SETITEMDATA, dwItem, dwItem);
						}
					}
				}
				SendMessage(hWndCombo, CB_SETCURSEL, 0, 0);
				MEMFREE(lpRasEntryName);
			}
			break;
		case IDC_BUTTON_CONS_NEW:
			{
				WCHAR szEntryName[RAS_MaxEntryName + 1];
				DWORD dwConnType, dwError;
				RASENTRY reEntry = { 0 };

				lstrcpyn(szEntryName, TEXT("New connection"), RAS_MaxEntryName);
				if (AskForConnName(hWnd, szEntryName) != NO_ERROR)
					break;
				dwConnType = MessageBox(hWnd,
				    TEXT("Chose connection type:\r\nYes - VPN connection\r\nNo - PPPoE connection\r\n"),
				    TEXT("Connection type"), (MB_YESNOCANCEL | MB_ICONQUESTION));
				if (dwConnType != IDYES && dwConnType != IDNO)
					break;
				reEntry.dwSize = sizeof(RASENTRY);
				reEntry.dwfNetProtocols = RASNP_Ip;
				reEntry.dwFramingProtocol = RASFP_Ppp;
				reEntry.dwfOptions = RASEO_IpHeaderCompression | RASEO_RemoteDefaultGateway | RASEO_ModemLights | RASEO_SwCompression | RASEO_PreviewUserPw;
				reEntry.dwfOptions2 = RASEO2_SecureFileAndPrint | RASEO2_SecureClientForMSNet | RASEO2_Internet | RASEO2_DisableNbtOverIP | RASEO2_ReconnectIfDropped;
				if (dwConnType == IDYES) { // vpn
					lstrcpyn(reEntry.szDeviceType, RASDT_Vpn, RAS_MaxDeviceType);
					reEntry.dwType = RASET_Vpn;
				} else {// PPPoE
					lstrcpyn(reEntry.szDeviceType, RASDT_PPPoE, RAS_MaxDeviceType);
					reEntry.dwType = RASET_Broadband;
				}
						
				dwError = RasSetEntryProperties(NULL, szEntryName, &reEntry, sizeof(RASENTRY), NULL, 0);
				if (dwError == NO_ERROR) { //  /
					RASENTRYDLG reDlg = { 0 };

					reDlg.dwSize = sizeof(RASENTRYDLG);
					reDlg.hwndOwner = hWnd;
					//reDlg.dwFlags = ;
					RasEntryDlg(NULL, szEntryName, &reDlg);
					SendMessage(hWnd, WM_COMMAND, IDC_BUTTON_CONS_REFRESH, 0);
				} else {
					ReportError(dwError);
				}
			}
			break;
		case IDC_BUTTON_CONS_COPY:
			{
				WCHAR szEntryName[RAS_MaxEntryName + 1];
				WCHAR szNewEntryName[RAS_MaxEntryName + 1];
				DWORD dwEntryInfoSize, dwError;
				RASENTRY reEntry = { 0 };
				
				if (!GetDlgItemTextW(hWnd, IDC_COMBO_CONNECTIONS, szEntryName, SIZEOF(szEntryName)))
					break;
				lstrcpyn(szNewEntryName, szEntryName, RAS_MaxEntryName);
				if (AskForConnName(hWnd, szNewEntryName) != NO_ERROR)
					break;
				reEntry.dwSize = sizeof(RASENTRY);
				dwEntryInfoSize = sizeof(RASENTRY);

				dwError = RasGetEntryProperties(NULL, szEntryName, &reEntry, &dwEntryInfoSize, NULL, NULL);
				if (dwError == NO_ERROR)
					dwError = RasSetEntryProperties(NULL, szNewEntryName, &reEntry, sizeof(RASENTRY), NULL, 0);
				if (dwError == NO_ERROR) { //  /
					RASENTRYDLG reDlg = { 0 };

					reDlg.dwSize = sizeof(RASENTRYDLG);
					reDlg.hwndOwner = hWnd;
					//reDlg.dwFlags = ;
					RasEntryDlg(NULL, szEntryName, &reDlg);

					SendMessage(hWnd, WM_COMMAND, IDC_BUTTON_CONS_REFRESH, 0);
				} else {
					ReportError(dwError);
				}
			}
			break;
		case IDC_BUTTON_CONS_EDIT:
			{
				WCHAR szEntryName[RAS_MaxEntryName + 1];
				RASENTRYDLG reDlg = { 0 };

				if (!GetDlgItemTextW(hWnd, IDC_COMBO_CONNECTIONS, szEntryName, SIZEOF(szEntryName)))
					break;
				reDlg.dwSize = sizeof(RASENTRYDLG);
				reDlg.hwndOwner = hWnd;
				//reDlg.dwFlags = ;
				RasEntryDlg(NULL, szEntryName, &reDlg);
			}
			break;
		case IDC_BUTTON_CONS_RENAME:
			{
				WCHAR szEntryName[RAS_MaxEntryName + 1];
				WCHAR szNewEntryName[RAS_MaxEntryName+1];
				
				if (!GetDlgItemTextW(hWnd, IDC_COMBO_CONNECTIONS, szEntryName, SIZEOF(szEntryName)))
					break;
				lstrcpyn(szNewEntryName, szEntryName, RAS_MaxEntryName);
				if (AskForConnName(hWnd, szNewEntryName) != NO_ERROR)
					break;
				RasRenameEntry(NULL, szEntryName, szNewEntryName);
				SendMessage(hWnd, WM_COMMAND, IDC_BUTTON_CONS_REFRESH, 0);
			}
			break;
		case IDC_BUTTON_CONS_DELETE:
			{
				WCHAR szEntryName[RAS_MaxEntryName + 1];
				
				if (!GetDlgItemTextW(hWnd, IDC_COMBO_CONNECTIONS, szEntryName, SIZEOF(szEntryName)))
					break;
				if (MessageBox(hWnd, TEXT("Are you sure?"), TEXT("Deleting connection"),
				    (MB_YESNO | MB_DEFBUTTON2 | MB_ICONQUESTION)) != IDYES)
					break;
				RasDeleteEntry(NULL, szEntryName);
				SendMessage(hWnd, WM_COMMAND, IDC_BUTTON_CONS_REFRESH, 0);
			}
			break;
		case IDC_COMBO_CONNECTIONS:
			if (HIWORD(wParam) == CBN_SELCHANGE) {
				WCHAR szEntryName[RAS_MaxEntryName + 1];
				SIZE_T dwEntryNameSize;
				
				dwEntryNameSize = GetDlgItemTextW(hWnd, IDC_COMBO_CONNECTIONS, szEntryName, SIZEOF(szEntryName));
			}
			break;
		}
		break;
	}
	return (FALSE);
}


DWORD
AskForConnName(HWND hWndParent, LPWSTR lpszName) {
	DWORD dwRet = ERROR_INVALID_HANDLE;

	if (NULL == lpszName)
		return(dwRet);
	dwRet = DialogBoxParam(psProcessSettings.hInstance, MAKEINTRESOURCE(IDD_DLG_CONN_NAME),
	    hWndParent, WndProcGetConnName, (LPARAM)lpszName);
	switch (dwRet) {
	case 0:
	case -1:
		dwRet = GetLastError();
		break;
	}

	return (dwRet);
}


INT_PTR CALLBACK
WndProcGetConnName(HWND hWnd,UINT message,WPARAM wParam,LPARAM lParam) {

	switch (message) {
	case WM_INITDIALOG:
		if (lParam) {
			SET_WINDOW_DATA(hWnd, lParam);
			SetDlgItemTextW(hWnd, IDC_EDIT_CONN_NAME, (LPWSTR)lParam);
		}
		break;
	case WM_CLOSE:
	case WM_QUIT:
		EndDialog(hWnd, ERROR_CANCELLED);
		DestroyWindow(hWnd);
		break;
	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDOK:
			{
				LPWSTR lpszEntryName;
				
				lpszEntryName = (LPWSTR)GET_WINDOW_DATA(hWnd);
				if (lpszEntryName)
					GetDlgItemTextW(hWnd, IDC_EDIT_CONN_NAME, lpszEntryName, RAS_MaxEntryName);
			}
			EndDialog(hWnd, NO_ERROR);
			break;
		case IDCANCEL:
			EndDialog(hWnd, ERROR_CANCELLED);
			break;
		case IDC_EDIT_CONN_NAME:
			if (HIWORD(wParam) == EN_CHANGE) {
				WCHAR szEntryName[RAS_MaxEntryName + 1];
				
				if (GetDlgItemTextW(hWnd, IDC_EDIT_CONN_NAME, szEntryName, SIZEOF(szEntryName))) {
					EnableWindow(GetDlgItem(hWnd, IDOK),
					    (RasValidateEntryName(NULL, szEntryName) == NO_ERROR));
				} else {
					EnableWindow(GetDlgItem(hWnd, IDOK), FALSE);
				}
			}
		}
		break;
	}
	return (FALSE);
}


DWORD
GetConnectionsList(LPRASENTRYNAME *plpRasEntryName, SIZE_T *pdwCount) {
	DWORD dwEntries, dwCb;
	RASENTRYNAME reRasEntryName;
	LPRASENTRYNAME lpRasEntryName;
	DWORD dwError;

	if (NULL == plpRasEntryName || NULL == pdwCount)
		return (ERROR_INVALID_HANDLE);

	/* Call RasEnumEntries with lpRasEntryName = NULL. dwCb is returned with the required buffer size and 
	 * a return code of ERROR_BUFFER_TOO_SMALL */
	dwCb = sizeof(RASENTRYNAME);
	dwEntries = 0;
	memset(&reRasEntryName, 0, sizeof(reRasEntryName));
	/* The first RASENTRYNAME structure in the array must contain the structure size */
	reRasEntryName.dwSize = sizeof(RASENTRYNAME);
	dwError = RasEnumEntries(NULL, NULL, &reRasEntryName, &dwCb, &dwEntries);
	if (dwError == ERROR_BUFFER_TOO_SMALL) /* Allocate the memory needed for the array of RAS entry names. */
		dwCb += sizeof(RASENTRYNAME);
	if (0 == dwCb) /* No enryes found. */
		return (ERROR_NO_MORE_ITEMS);
	lpRasEntryName = (LPRASENTRYNAME)MEMALLOC(dwCb);
	if (NULL == lpRasEntryName)
		return (GetLastError());
	if (dwError == NO_ERROR) { /* Just copy and return. */
		memcpy(lpRasEntryName, &reRasEntryName, sizeof(reRasEntryName));
	} else {
		/* The first RASENTRYNAME structure in the array must contain the structure size */
		lpRasEntryName[0].dwSize = sizeof(RASENTRYNAME);
		/* Call RasEnumEntries to enumerate all RAS entry names */
		dwError = RasEnumEntries(NULL, NULL, lpRasEntryName, &dwCb, &dwEntries);
		/* If successful, print the RAS entry names */
		if (dwError != NO_ERROR) {
			/* Deallocate memory for the connection buffer */
			MEMFREE(lpRasEntryName);
			return (dwError);
		}
	}
	(*plpRasEntryName) = lpRasEntryName;
	(*pdwCount) = dwEntries;

	return (NO_ERROR);
}

